// エモリングを使ったイヤリング
#include <Adafruit_NeoPixel.h>
#include "Fluctuator.h"

#define     LED_PIN     1 // LED(NeoPixel)の制御ピン
#define     LED_MAX     12 // LED(NeoPixel)の数
#define     WAIT1_MS     30 // 30 ウェイト[msec]
#define     WAIT2_MS     80 // 80 ウェイト[msec]
#define     WAIT3_MS     50 // 50 ウェイト[msec]

// 番号
enum{
  RED = 0,
  GREEN,
  BLUE,
  RGB
};

enum{
  BRIGHT = 0, // 輝度
  HUE,        // 色相
  BRIGHT_HUE
};

// 基準色
const int COLOR0[RGB]={0x00,0x00,0xFF};
const int COLOR1[RGB]={0x00,0x00,0x40};
//const int COLOR2[RGB]={0x10,0x20,0x40};
const int COLOR2[RGB]={0x40,0x40,0x40};

// LED(NeoPixel)制御器
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(LED_MAX, LED_PIN, NEO_GRB + NEO_KHZ800);

// 1/fゆらぎ発生器 (各LED×{輝度、色相})
Fluctuator fluct[LED_MAX][BRIGHT_HUE];

// 初期化
void setup()
{
    pinMode(0,OUTPUT);
    digitalWrite(0,LOW);
    delay(100);
    
    // LED(NeoPixel)の初期化
    pixels.begin();
    pixels.setBrightness(50);
    
    // ゆらぎ発生器の初期化
    for(int i=0;i<LED_MAX;i++){
        for(int j=0;j<RGB;j++){
            fluct[i][j].init();
        }
    }
}

// メインループ
void loop()
{
//  loop_2();

  for(int i=0;i<400;i++) loop_1();
  for(int i=0;i<12*10;i++) loop_2();
  for(int i=0;i<400;i++) loop_3();
}

// ゆらゆら
void loop_1()
{
    // 色の変化(2つの基準色の間を行ったり来たり)
    long color[RGB];
    static int ratio = 0;
    static int incdec = 1;
    ratio += incdec;
    if(ratio > 100){
        ratio = 100;
        incdec = -1;
    }else if(ratio < 0){
        ratio = 0;
        incdec = 1;
    }
    for(int i=0;i<RGB;i++){
        color[i] = (COLOR0[i]*ratio + COLOR2[i]*(100-ratio)) / 100;
    }
    
    // 各LEDについて
    for(int i=0;i<LED_MAX;i++){
        // ゆらぎの計算
        float fb = fluct[i][BRIGHT].calc(); // 輝度のゆらぎ
        float fh = fluct[i][HUE].calc();    // 色相のゆらぎ
        // LEDの色計算
        float kb = 0.6 + 0.4 * fb;      // 輝度のゆらぎは基準色に対し80～100%の範囲で
        float kh = 0.2 * fh - 0.1;     // 色相のゆらぎは基準色に対し-10～+10%の範囲で
        int r = (int)((float)color[RED]   * (kb+kh));
        int g = (int)((float)color[GREEN] * (kb-kh));
        int b = (int)((float)color[BLUE]  *  kb);
        if(r>255) r=255; else if(r<0) r=0;
        if(g>255) g=255; else if(g<0) g=0;
        if(b>255) b=255; else if(b<0) b=0;
        // LEDの色設定
        pixels.setPixelColor(i, pixels.Color(r,g,b));
    }
    // LEDの色更新
    pixels.show();
    // ウェイト
    delay(WAIT1_MS);
}

// ぐるぐる
void loop_2()
{
    static int offset = 0;
    offset++;
    if(offset>=12){
      offset = 0;
    }
    
    // 各LEDについて
    long color[RGB];
    for(int i=0;i<LED_MAX;i++){
      int ratio = (offset + i) % 12;
      for(int j=0;j<RGB;j++){
          color[j] = (COLOR1[j]*ratio + COLOR2[j]*(12-ratio)) / 12;
      }
      int r = (int)color[RED];
      int g = (int)color[GREEN];
      int b = (int)color[BLUE];
      if(r>255) r=255; else if(r<0) r=0;
      if(g>255) g=255; else if(g<0) g=0;
      if(b>255) b=255; else if(b<0) b=0;
        
      // LEDの色設定
      pixels.setPixelColor(i, pixels.Color(r,g,b));
    }
    // LEDの色更新
    pixels.show();
    // ウェイト
    delay(WAIT2_MS);
}

// ちかちか
void loop_3()
{
    // 色の変化(2つの基準色の間を行ったり来たり)
    long color[RGB];
    static int ratio = 0;
    static int incdec = 1;
    ratio += incdec;
    if(ratio > 100){
        ratio = 100;
        incdec = -1;
    }else if(ratio < 0){
        ratio = 0;
        incdec = 1;
    }
    for(int i=0;i<RGB;i++){
        color[i] = (COLOR0[i]*ratio + COLOR2[i]*(100-ratio)) / 100;
    }

    static int parity = 0;
    parity++;
    parity %= 3;
    
    // 各LEDについて
    for(int i=0;i<LED_MAX;i++){
        int r = (int)color[RED];
        int g = (int)color[GREEN];
        int b = (int)color[BLUE];
        if(r>255) r=255; else if(r<0) r=0;
        if(g>255) g=255; else if(g<0) g=0;
        if(b>255) b=255; else if(b<0) b=0;
        // LEDの色設定
        if((i % 3) != parity){
          r /= 5;
          g /= 5;
          b /= 5;
        }
        pixels.setPixelColor(i, pixels.Color(r,g,b));
    }
    // LEDの色更新
    pixels.show();
    // ウェイト
    delay(WAIT3_MS);
}
